#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V2;
using System.IO;
using System.Text;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "Reader", Category = "File", Version = "Fix", Help = "Reads files, outputs content as string", Author="bjo:rn",  Tags = "")]
	#endregion PluginInfo
	public class FileReaderNode : IPluginEvaluate
	{
	#region fields & pins
		[Input("Filename", StringType = StringType.Filename)]
		ISpread<string> FInput;
		
		[Input("Encoding", EnumName = "CodePages")]
		IDiffSpread<EnumEntry> FEncoding;

		[Input("Read", IsBang = true)]
		ISpread<bool> FRead;
		
		[Output("Content")]
		ISpread<string> FContent;
		
		Spread<string> path = new Spread<string>(0);
		Spread<FileStream> fileStream = new Spread<FileStream>(0);
		
		[Import()]
        public ILogger FLogger;
		
		#endregion fields & pins

		//called when data for any output pin is requested
		public void Evaluate(int spreadMax)
		{
			FContent.SliceCount = spreadMax;

			
			path.ResizeAndDismiss(spreadMax, () => string.Empty);

			fileStream.Resize(spreadMax, (i) => new FileStream(FInput[i], FileMode.Open, FileAccess.Read, System.IO.FileShare.ReadWrite), (t) => t.Dispose());
			
			for (int i = 0; i < spreadMax; i++)
			{

				bool update = false;
				bool isValid = true;
				if (path[i] != FInput[i] || FRead[i])
				{
					try
					{
						fileStream[i].Dispose(); 
						fileStream[i] = new FileStream(FInput[i], FileMode.Open, FileAccess.Read, System.IO.FileShare.ReadWrite);
						path[i] = FInput[i];
						update = true;
					}
					catch
					{
						FContent[i]=string.Empty;
						isValid = false;
						//FLogger.Log(LogType.Debug, "hi tty!");
					}
				}
				
				//do the reading part
				if (update && isValid)
				{
					System.Text.Encoding enc = System.Text.Encoding.Default;
		
					if (FEncoding[i].Index > 0) enc = Encoding.GetEncoding(FEncoding[i].Name);
					
					StreamReader streamReader = new StreamReader(fileStream[i],enc);
					FContent[i] = streamReader.ReadToEnd();
					streamReader.Close();
				}
			}
		}
	}
}
